/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Swedish Institute
 *      of Computer Science and its contributors.
 * 4. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: httpd.c,v 1.2 2001/06/21 09:43:30 adam Exp $
 */

#include <string.h>
#include <stdio.h>

#include "lwip/mem.h"
#include "lwip/debug.h"
#include "lwip/def.h"
#include "lwip/api.h"
#include "lwip/stats.h"

#include "fs.h"
#include "dynpages.c"
static const char httphtmlheader[] = "HTTP/1.0 200 OK\r\nContent-Type: text/html\r\n\r\n";
static char statsbuf[631];

/*-----------------------------------------------------------------------------------*/
static void
hexprint(u16_t *databytes, u16_t num, char *buf)
{
  static u8_t space;
  static u16_t i, j;
  static u16_t a;
  static const char hextab[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
			        'a', 'b', 'c', 'd', 'e', 'f'};

  for(i = 0, j = 0; i < num; i++, j += 5) {
    a = databytes[i];
    if(((a >> 12) & 0x0f) != 0) {
      buf[j + 0] = hextab[(a >> 12) & 0x0f];
      space = 0;      
    } else {
      buf[j + 0] = ' ';
      space = 1;      
    }
    if(((a >> 8) & 0x0f) != 0 || !space) {
      buf[j + 1] = hextab[(a >> 8) & 0x0f];
      space = 0;      
    } else {
      buf[j + 1] = ' ';
      space = 1;      
    }    
    if(((a >> 4) & 0x0f) != 0 || !space) {
      buf[j + 2] = hextab[(a >> 4) & 0x0f];
      space = 0;      
    } else {
      buf[j + 2] = ' ';
      space = 1;      
    }    
    if(((a >> 0) & 0x0f) != 0 || !space) {
      buf[j + 3] = hextab[(a >> 0) & 0x0f];
    } else {
      buf[j + 3] = ' ';
    }    

        
    buf[j + 4] = '\n';
  }
  
}
/*-----------------------------------------------------------------------------------*/
static void
http_input(struct netconn *conn)
{
  static int i, j;
  static struct netbuf *buf;
  static char *data;
  static int len;
  static char fname[40];
  static struct fs_file file;
  
  buf = netconn_recv(conn);
  if(buf != NULL) {
    netbuf_data(buf, (void *)&data, &len);
    if(strncmp(data, "GET ", 4) == 0) {
      for(i = 0; i < 40; i++) {
	if(((char *)data + 4)[i] == ' ' ||
	   ((char *)data + 4)[i] == '\r' ||
	   ((char *)data + 4)[i] == '\n') {
	  ((char *)data + 4)[i] = 0;
	}
      }
      i = 0;
      do {
	fname[i] = "/http"[i];
	i++;
      } while(fname[i - 1] != 0 && i < 40);
      i--;
      j = 0;
      do {
	fname[i] = ((char *)data + 4)[j];
	j++;
	i++;
      } while(fname[i - 1] != 0 && i < 40);           
      netbuf_free(buf);      
      if(fs_open(fname, &file)) {
	netconn_write(conn, file.data, file.len, NETCONN_NOCOPY);         
      } else {
        if(strncmp(fname, "/http/stats.html", 15) == 0) {
          netconn_write(conn, (void *)httphtmlheader,
			sizeof(httphtmlheader) - 1, NETCONN_NOCOPY);
          netconn_write(conn, (void *)statshtmlheader,
			sizeof(statshtmlheader) - 1, NETCONN_NOCOPY);
#ifdef STATS
	  
	  hexprint((u16_t *)&stats,
		   (u16_t)(sizeof(struct stats) / sizeof(u16_t)),
		   statsbuf);
	  netconn_write(conn, (void *)statsbuf,
			sizeof(statsbuf) - 1, NETCONN_NOCOPY);
#endif /* STATS */
          netconn_write(conn, (void *)statshtmlfooter,
			sizeof(statshtmlfooter), NETCONN_NOCOPY);
        } else if(fs_open("/http/index.html", &file)) {
          netconn_write(conn, file.data, file.len, NETCONN_NOCOPY);         
        }
      }
    }
    netbuf_delete(buf);
  }
  netconn_close(conn);
}
/*-----------------------------------------------------------------------------------*/
static void *
httpd_thread(void *arg)
{
  static struct netconn *conn, *newconn;
  
  conn = netconn_new(NETCONN_TCP);
  netconn_bind(conn, NULL, 90);
  netconn_listen(conn);

  while(1) {
    newconn = netconn_accept(conn);
    http_input(newconn);
    netconn_delete(newconn);
  }
}
/*-----------------------------------------------------------------------------------*/
void
httpd_init(void)     
{
  sys_thread_new(httpd_thread, NULL);
}



