/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met: 
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright 
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution. 
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the Swedish Institute
 *      of Computer Science and its contributors.
 * 4. Neither the name of the Institute nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE. 
 *
 * This file is part of the lwIP TCP/IP stack.
 * 
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: memp.c,v 1.8 2001/06/21 12:58:10 adam Exp $
 */

#include "lwipopts.h"

#include "lwip/memp.h"

#include "lwip/pbuf.h"
#include "lwip/udp.h"
#include "lwip/tcp.h"
#include "lwip/api.h"
#include "lwip/api_msg.h"
#include "lwip/tcpip.h"

#include "lwip/sys.h"
#include "lwip/stats.h"

#define MEMP_FLAG_NONE 0x00
#define MEMP_FLAG_USED 0x01

struct memp {
#if MEMP_DEBUG
  char deadbeef[4];
  memp_t type;
#endif /* MEMP_DEBUG */
  struct memp *next;
};

#define MEMP_ALLOC(mem, type)     mem = memp_tab[type]; \
                                  if(mem != NULL) { \
                                     memp_tab[type] = mem->next; \
                                     mem->next = NULL; }

#define MEMP_FREE(mem, type)      mem->next = memp_tab[type]; \
                                  memp_tab[type] = mem;


static struct memp *memp_tab[MEMP_MAX];

static const u16_t memp_sizes[MEMP_MAX] = {
  sizeof(struct pbuf),
  sizeof(struct udp_pcb),
  sizeof(struct tcp_pcb),
  sizeof(struct tcp_pcb_listen),
  sizeof(struct tcp_pcb_tw),
  sizeof(struct tcp_seg),
  sizeof(struct netbuf),
  sizeof(struct netconn),
  sizeof(struct api_msg),
  sizeof(struct tcpip_msg)
};

static const u8_t memp_num[MEMP_MAX] = {
  MEMP_NUM_PBUF,
  MEMP_NUM_UDP_PCB,
  MEMP_NUM_TCP_PCB,
  MEMP_NUM_TCP_PCB_LISTEN,
  MEMP_NUM_TCP_PCB_TW, 
  MEMP_NUM_TCP_SEG,
  MEMP_NUM_NETBUF,
  MEMP_NUM_NETCONN,
  MEMP_NUM_API_MSG,
  MEMP_NUM_TCPIP_MSG
};

#if MEMP_RECLAIM
struct memp_reclaim {
  struct memp_reclaim *next;
  memp_reclaim_func f;
  void *arg;  
};

static char memp_memory[MEMP_NUM_PBUF * (sizeof(struct pbuf) + sizeof(struct memp)) +
		       MEMP_NUM_UDP_PCB * (sizeof(struct udp_pcb) + sizeof(struct memp)) +
		       MEMP_NUM_TCP_PCB * (sizeof(struct tcp_pcb) + sizeof(struct memp)) +
		       MEMP_NUM_TCP_PCB_LISTEN * (sizeof(struct tcp_pcb_listen) + sizeof(struct memp)) +
		       MEMP_NUM_TCP_PCB_TW * (sizeof(struct tcp_pcb_tw) + sizeof(struct memp)) +
		       MEMP_NUM_TCP_SEG * (sizeof(struct tcp_seg) + sizeof(struct memp)) +
		       MEMP_NUM_NETBUF * (sizeof(struct netbuf) + sizeof(struct memp)) +
		       MEMP_NUM_NETCONN * (sizeof(struct netconn) + sizeof(struct memp)) +
		       MEMP_NUM_API_MSG * (sizeof(struct api_msg) + sizeof(struct memp)) +
		       MEMP_NUM_TCPIP_MSG * (sizeof(struct tcpip_msg) + sizeof(struct memp))];
		       

static struct memp_reclaim *memp_reclaim_funcs[MEMP_MAX];

static u8_t memp_reclaim(memp_t type);
#endif /* MEMP_RECLAIM */

#if MEMP_DEBUG
static struct memp *memp_lowest, *memp_highest;
#endif /* MEMP_DEBUG */
/*-----------------------------------------------------------------------------------*/
static sys_sem_t mutex;
/*-----------------------------------------------------------------------------------*/
void
memp_init(void)
{
  struct memp *m, *memp;
  int i, j;
  u16_t size;
      
#ifdef MEMP_STATS
  for(i = 0; i < MEMP_MAX; i++) {
    stats.memp[i].used = stats.memp[i].max =
      stats.memp[i].err = stats.memp[i].reclaimed = 0;
    stats.memp[i].avail = memp_num[i];
  }
#endif /* MEMP_STATS */

#if MEMP_DEBUG
  memp_lowest = (struct memp *)&memp_memory[0];
  memp_highest = (struct memp *)&memp_memory[sizeof(memp_memory)];
#endif /* MEMP_DEBUG */

  memp = (struct memp *)&memp_memory[0];
  for(i = 0; i < MEMP_MAX; i++) {
    size = (memp_sizes[i] + sizeof(struct memp));
    DEBUGF(MEMP_DEBUG, ("memp_init: mem pool %d size %d * %d\n",
			i, memp_sizes[i], memp_num[i]));
    if(memp_num[i] > 0) {
      memp_tab[i] = memp;
      m = memp;
      
#if MEMP_DEBUG
#endif /* MEMP_DEBUG */

      ASSERT("memp_init: enough memory to allocate mem pool", m != NULL);
      for(j = 0; j < memp_num[i]; j++) {
	m->next = (struct memp *)((char *)m + size);
#if MEMP_DEBUG
	strncpy(m->deadbeef, "memp", 4);
#endif /* MEMP_DEBUG */
	memp = m;
	m = m->next;
      }
      memp->next = NULL;
      memp = m;
#if MEMP_DEBUG
      m = memp_tab[i];
      for(j = 0; j < memp_num[i] - 1; j++) {
	ASSERT("memp_init: m->next != NULL", m->next != NULL);
	ASSERT("memp_free: m sane", m <= memp_highest &&
	       m >= memp_lowest);
	m = m->next;
      }
      ASSERT("memp_init: m->next == NULL", m->next == NULL);
#endif /* MEMP_DEBUG */
    } else {
      memp_tab[i] = NULL;
    }
  }

  mutex = sys_sem_new(1);

#if MEMP_RECLAIM
  for(i = 0; i < MEMP_MAX; i++) {
    memp_reclaim_funcs[i] = NULL;
  }
#endif /* MEMP_RECLAIM */
  
}
/*-----------------------------------------------------------------------------------*/
void *
memp_malloc(memp_t type)
{
  struct memp *memp;
#if MEMP_DEBUG
  struct memp *dmemp;
#endif /* MEMP_DEBUG */

  ASSERT("memp_malloc: type < MEMP_MAX", type < MEMP_MAX);

  MEMP_ALLOC(memp, type);
  if(memp != NULL) {
#if MEMP_DEBUG
    ASSERT("memp_malloc: deadbeef", strncmp(memp->deadbeef, "memp", 4) == 0);
    
    for(dmemp = memp_tab[type]; dmemp != NULL; dmemp = dmemp->next) {
      ASSERT("memp_malloc: dmemp deadbeef", strncmp(dmemp->deadbeef, "memp", 4) == 0);
      ASSERT("memp_malloc: dmemp sane", dmemp <= memp_highest &&
	     dmemp >= memp_lowest);
    }
    memp->type = type;
    ASSERT("memp_malloc: deadbeef", strncmp(memp->deadbeef, "memp", 4) == 0);
#endif /* MEMP_DEBUG */
#ifdef MEMP_STATS
    stats.memp[type].used++;
    if(stats.memp[type].used > stats.memp[type].max) {
      stats.memp[type].max = stats.memp[type].used;
    }
#endif /* MEMP_STATS */   
    return (char *)memp + sizeof(struct memp);
  } else {
    DEBUGF(MEMP_DEBUG, ("memp_malloc: out of memory in pool %d\n", type));
#ifdef MEMP_STATS
    stats.memp[type].err++;
#endif /* MEMP_STATS */
    return NULL;
  }
}
/*-----------------------------------------------------------------------------------*/
void *
memp_mallocp(memp_t type)
{
  void *mem;
  sys_sem_wait(mutex);
  mem = memp_malloc(type);
  sys_sem_signal(mutex);
  return mem;
}
/*-----------------------------------------------------------------------------------*/
void *
memp_malloc2(memp_t type)
{
  void *mem;

  mem = memp_malloc(type);
#if MEMP_RECLAIM
  if(mem == NULL) {
    memp_reclaim(type);
    mem = memp_malloc(type);
  }
#endif /* MEMP_RECLAIM */
  return mem;
}
/*-----------------------------------------------------------------------------------*/
void *
memp_realloc(memp_t fromtype, memp_t totype, void *mem)
{
  void *rmem;
  u16_t size;
  
  DEBUGF(MEMP_DEBUG, ("memp_realloc: from %d to %d\n", fromtype, totype));

  if(mem == NULL) {
    return NULL;
  }
  
  rmem = memp_malloc(totype);
  if(rmem != NULL) { 
    size = memp_sizes[totype];
    if(memp_sizes[fromtype] < size) {
      size = memp_sizes[fromtype];
    }
    DEBUGF(MEMP_DEBUG, ("memp_realloc: copying %d bytes\n", size));
    bcopy(mem, rmem, size);
    memp_free(fromtype, mem);
  }
  return rmem;
}
/*-----------------------------------------------------------------------------------*/
void
memp_free(memp_t type, void *mem)
{
  struct memp *memp;
#if MEMP_DEBUG
  struct memp *dmemp;
#endif /* MEMP_DEBUG */
  if(mem == NULL) {
    return;
  }
  memp = (struct memp *)((char *)mem - sizeof(struct memp));

#if MEMP_DEBUG
  ASSERT("memp_free: mem sane", memp <= memp_highest &&
	 memp >= memp_lowest);
#endif /* MEMP_DEBUG */

#ifdef MEMP_STATS
  stats.memp[type].used--; 
#endif /* MEMP_STATS */

#if MEMP_DEBUG
  ASSERT("memp_free: memp->next == NULL", memp->next == NULL);
  ASSERT("memp_free: deadbeef", strncmp(memp->deadbeef, "memp", 4) == 0);
  ASSERT("memp_free: types match", memp->type == type);  
  for(dmemp = memp_tab[type]; dmemp != NULL; dmemp = dmemp->next) {
    ASSERT("memp_free: dmemp deadbeef", strncmp(dmemp->deadbeef, "memp", 4) == 0);
    ASSERT("memp_free: dmemp sane", dmemp <= memp_highest &&
	   dmemp >= memp_lowest);
  }
#endif /* MEMP_DEBUG */
  
  MEMP_FREE(memp, type);
#if MEMP_DEBUG
  for(dmemp = memp_tab[type]; dmemp != NULL; dmemp = dmemp->next) {
    ASSERT("memp_free: dmemp deadbeef", strncmp(dmemp->deadbeef, "memp", 4) == 0);
    ASSERT("memp_free: dmemp sane", dmemp <= memp_highest &&
	   dmemp >= memp_lowest);
  }
#endif /* MEMP_DEBUG */
  return;
}
/*-----------------------------------------------------------------------------------*/
void 
memp_freep(memp_t type, void *mem)
{
  sys_sem_wait(mutex);
  memp_free(type, mem);
  sys_sem_signal(mutex);
}
/*-----------------------------------------------------------------------------------*/
static u8_t
memp_reclaim(memp_t type)
{
  struct memp_reclaim *mr;
  
  
  for(mr = memp_reclaim_funcs[type]; mr != NULL; mr = mr->next) {
    DEBUGF(MEMP_DEBUG, ("memp_reclaim: calling reclaimer for type %d\n", type));
    if(mr->f(mr->arg, type) != 0) {
#ifdef MEMP_STATS
      stats.memp[type].reclaimed++;
#endif /* MEMP_STATS */
      return 1;
    }
  }
  return 0;
}
/*-----------------------------------------------------------------------------------*/
void
memp_register_reclaim(memp_t type, memp_reclaim_func f, void *arg)
{
  struct memp_reclaim *mr;

  mr = mem_malloc(sizeof(struct memp_reclaim));
  mr->next = memp_reclaim_funcs[type];
  memp_reclaim_funcs[type] = mr;  
  mr->f = f;
  mr->arg = arg;
  DEBUGF(MEMP_DEBUG, ("memp_register_reclaim: registering reclaimer for type %d\n", type));
}
/*-----------------------------------------------------------------------------------*/
